package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.scene.Target;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.random.RandomGenerator;

/**
 * A probability density function targeting a target.
 * @see Target
 */
public final class TargetingProbabilityDensityFunction implements ProbabilityDensityFunction {
    private final @NotNull Vec3 origin;
    private final @NotNull List<@NotNull Target> targets;

    public TargetingProbabilityDensityFunction(@NotNull Vec3 origin, @NotNull List<@NotNull Target> targets) {
        this.origin = Objects.requireNonNull(origin, "origin");
        this.targets = new ArrayList<>(targets);
    }

    @Override
    public double value(@NotNull Vec3 direction) {
        var weight = 1d / targets.size();
        var sum = 0.0;

        for (var target : targets) {
            sum += weight * target.getProbabilityDensity(origin, direction);
        }

        return sum;
    }

    @Override
    public @NotNull Vec3 generate(@NotNull RandomGenerator random) {
        return targets.get(random.nextInt(targets.size())).getTargetingDirection(origin, random);
    }
}
