package eu.jonahbauer.raytracing.render.texture;

import eu.jonahbauer.raytracing.math.Vec3;
import eu.jonahbauer.raytracing.scene.HitResult;
import org.jetbrains.annotations.NotNull;

public interface Texture {
    /**
     * {@return the color of <code>this</code> texture for a hit}
     */
    default @NotNull Color get(@NotNull HitResult hit) {
        return get(hit.u(), hit.v(), hit.position());
    }

    /**
     * {@return the color of <code>this</code> texture at the specified position}
     * @param u the texture u coordinate
     * @param v the texture v coordinate
     * @param p the position
     */
    @NotNull Color get(double u, double v, @NotNull Vec3 p);

    /**
     * Returns whether {@link #get(double, double, Vec3)} uses the {@code u} and/or {@code v} parameters.
     * When a texture indicates that the {@code u} and {@code v} coordinates are not required, the calculation may be
     * skipped and {@link Double#NaN} will be passed.
     * @return whether {@link #get(double, double, Vec3)} uses the {@code u} and/or {@code v} parameters
     */
    default boolean isUVRequired() {
        return true;
    }
}
